import ajenti
from ajenti.api import *
from ajenti.plugins import manager
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder
from ajenti.users import UserManager, PermissionProvider, restrict
from reconfigure.items.ajenti import UserData

from api import UserHierarchyManager


@plugin
class UserHierarchySection (SectionPlugin):
    def init(self):
        self.title = 'Multi-tenant'
        self.icon = 'group'
        self.category = ''
        self.order = 70

        self.append(self.ui.inflate('custom_userhierarchy:main'))

        self.hmgr = UserHierarchyManager.get(manager.context)
        self.binder = Binder(self, self)

        self.find('users').new_item = lambda c: self.create_user()

        def post_user_bind(object, collection, item, ui):
            box = ui.find('permissions')
            box.empty()

            p = PermissionProvider.get_all()
            for prov in p:
                line = self.ui.create('tab', title=prov.get_name())
                box.append(line)
                for perm in prov.get_permissions():
                    if UserManager.get().has_permission(self.context, perm[0]):
                        line.append(
                            self.ui.create(
                                'checkbox', id=perm[0],
                                text=perm[1],
                                value=(perm[0] in item.permissions)
                            )
                        )

        self.find('users').post_item_bind = post_user_bind

        def post_user_update(object, collection, item, ui):
            box = ui.find('permissions')
            for prov in PermissionProvider.get_all():
                for perm in prov.get_permissions():
                    if UserManager.get().has_permission(self.context, perm[0]):
                        has = box.find(perm[0]).value
                        if has and not perm[0] in item.permissions:
                            item.permissions.append(perm[0])
                        if not has and perm[0] in item.permissions:
                            item.permissions.remove(perm[0])
            if ui.find('password').value:
                item.password = ui.find('password').value

        self.find('users').post_item_update = post_user_update

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.binder.unpopulate()
        self.users = filter(
            lambda i:
            self.hmgr.get_parent(i.name) == self.context.session.identity,
            ajenti.config.tree.users.values()
        )
        self.binder.setup().populate()

    @on('new-user', 'click')
    def create_user(self):
        self.save()
        u = UserData()
        u.name = self.find('new-user-name').value
        for k, v in ajenti.config.tree.users.iteritems():
            if k == u.name:
                self.context.notify(
                    'error',
                    'Username "%s" is already taken' % u.name
                )
                return
        ajenti.config.tree.users.update({u.name: u})
        self.hmgr.set_parent(u.name, self.context.session.identity)
        self.hmgr.set_organization(u.name, self.hmgr.get_organization(self.context.session.identity))
        self.refresh()

    @on('save-button', 'click')
    def save(self):
        self.binder.update()

        for user in self.users:
            ajenti.config.tree.users.update({user.name: user})
            if not '|' in user.password:
                user.password = UserManager.get(manager.context) \
                    .hash_password(user.password)

        self.refresh()
        ajenti.config.save()
        self.context.notify('info', _('Saved'))
